@@ -1,9 +1,10 @@
 //! Derive macros for easily translating between objects in Qualia and Rust structs.
 
 extern crate proc_macro;
+use convert_case::{Case, Casing};
 use proc_macro::TokenStream;
 use proc_macro2::TokenStream as TokenStream2;
-use quote::{quote, quote_spanned, ToTokens};
+use quote::{format_ident, quote, quote_spanned, ToTokens};
 use syn::{parse_macro_input, DeriveInput};
 
 macro_rules! error_stream {
@@ -34,7 +35,7 @@ fn parse_field_name(field: &syn::Field) -> syn::Result<String> {
     if let Some(attr) = field
         .attrs
         .iter()
-        .find(|attr| attr.style == syn::AttrStyle::Outer && attr.path.is_ident("object_field"))
+        .find(|attr| attr.style == syn::AttrStyle::Outer && attr.path.is_ident("field"))
     {
         attr.parse_args::<syn::LitStr>().map(|lit| lit.value())
     } else {
@@ -47,6 +48,8 @@ struct ParsedField {
     ident: proc_macro2::Ident,
     name: String,
     accessor: TokenStream2,
+    inserter: TokenStream2,
+    related_impl: Option<TokenStream2>,
 }
 
 fn base_accessor(field_name: &String) -> TokenStream2 {
@@ -87,6 +90,49 @@ fn string_accessor(field_name: &String) -> TokenStream2 {
     )
 }
 
+fn object_id_accessor() -> TokenStream2 {
+    quote!(get("object_id")
+        .map(
+            |f| f.as_number().ok_or(qualia::ConversionError::FieldWrongType(
+                "object_id".to_string(),
+                "number".to_string(),
+            ),)
+        )
+        .transpose()?)
+}
+
+#[allow(unused)]
+fn option_i64_path() -> syn::TypePath {
+    syn::parse_str("Option<i64>").unwrap()
+}
+
+fn related_impl(field: &syn::Field) -> syn::Result<Option<TokenStream2>> {
+    if let Some(attr) = field
+        .attrs
+        .iter()
+        .find(|attr| attr.style == syn::AttrStyle::Outer && attr.path.is_ident("related"))
+    {
+        let typ = attr.parse_args::<syn::TypePath>()?;
+        let field_ident = field.ident.clone();
+        let helper_base = field
+            .ident
+            .clone()
+            .unwrap()
+            .to_string()
+            .to_case(Case::Snake)
+            .replace("_id", "");
+        let fetch_name = format_ident!("fetch_{}", helper_base);
+
+        Ok(Some(quote!(
+            fn #fetch_name(&self, store: &qualia::Store) -> qualia::Result<#typ> where #typ: qualia::ObjectShapeWithId {
+                store.query(<#typ as qualia::ObjectShape>::q().id(self.#field_ident)).one_as()
+            }
+        )))
+    } else {
+        Ok(None)
+    }
+}
+
 fn parse_fields(
     named_fields: &syn::FieldsNamed,
 ) -> syn::Result<(Vec<ParsedField>, Option<syn::Ident>)> {
@@ -98,25 +144,33 @@ fn parse_fields(
             .iter()
             .map(|field| {
                 if let Some(_) = field.attrs.iter().find(|attr| {
-                    attr.style == syn::AttrStyle::Outer && attr.path.is_ident("object_rest_fields")
+                    attr.style == syn::AttrStyle::Outer && attr.path.is_ident("rest_fields")
                 }) {
                     rest_field_ident = Some(field.ident.clone().unwrap());
                     return Ok(None);
                 }
 
-                let field_type = match &field.ty {
-                    syn::Type::Path(p) => p,
-                    _ => {
-                        return Err(syn::Error::new_spanned(
+                let field_type =
+                    match &field.ty {
+                        syn::Type::Path(p) => p,
+                        _ => return Err(syn::Error::new_spanned(
                             &field.ty,
-                            "fields in ObjectShape must be i64 or String",
-                        ))
-                    }
-                };
+                            "fields in ObjectShape must be simple types (usually i64 or String)",
+                        )),
+                    };
 
                 let field_name = parse_field_name(&field)?;
 
-                let field_type_accessor = if field_type.path.is_ident("i64") {
+                let field_type_accessor = if field_name == "object_id" {
+                    if *field_type == option_i64_path() {
+                        Ok(object_id_accessor())
+                    } else {
+                        Err(syn::Error::new_spanned(
+                            &field_type.path,
+                            "object_id field of OptionShape must be Option<i64>",
+                        ))
+                    }
+                } else if field_type.path.is_ident("i64") {
                     Ok(number_accessor(&field_name))
                 } else if field_type.path.is_ident("String") {
                     Ok(string_accessor(&field_name))
@@ -127,10 +181,23 @@ fn parse_fields(
                     ))
                 }?;
 
+                let field_ident = field.ident.clone().unwrap();
+                let field_inserter = if field_name == "object_id" {
+                    quote!(
+                        if let Some(object_id) = self.#field_ident {
+                            result.insert("object_id".into(), object_id.into());
+                        }
+                    )
+                } else {
+                    quote!(result.insert(#field_name.into(), self.#field_ident.into());)
+                };
+
                 Ok(Some(ParsedField {
-                    ident: field.ident.clone().unwrap(),
+                    ident: field_ident,
                     name: field_name,
                     accessor: field_type_accessor,
+                    inserter: field_inserter,
+                    related_impl: related_impl(&field)?,
                 }))
             })
             .collect::<syn::Result<Vec<_>>>()?
@@ -209,9 +276,10 @@ impl syn::parse::Parse for FixedFields {
 }
 
 fn parse_fixed_fields(attrs: &Vec<syn::Attribute>) -> syn::Result<Vec<FixedField>> {
-    let attr = match attrs.iter().find(|attr| {
-        attr.style == syn::AttrStyle::Outer && attr.path.is_ident("object_fixed_fields")
-    }) {
+    let attr = match attrs
+        .iter()
+        .find(|attr| attr.style == syn::AttrStyle::Outer && attr.path.is_ident("fixed_fields"))
+    {
         Some(a) => a,
         None => return Ok(Vec::new()),
     };
@@ -270,7 +338,7 @@ fn parse_fixed_fields(attrs: &Vec<syn::Attribute>) -> syn::Result<Vec<FixedField
 /// # Renaming properties
 ///
 /// By default, properties get the same name as the field in the struct. This can be changed with
-/// the `object_field` attribute:
+/// the `field` attribute:
 ///
 /// ```
 /// # use qualia::{object, Object};
@@ -278,7 +346,7 @@ fn parse_fixed_fields(attrs: &Vec<syn::Attribute>) -> syn::Result<Vec<FixedField
 /// # use std::convert::{Infallible, TryFrom};
 /// #[derive(Debug, ObjectShape, PartialEq)]
 /// struct CustomShape {
-///     #[object_field("my-name")]
+///     #[field("my-name")]
 ///     name: String,
 ///     width: i64,
 /// }
@@ -297,7 +365,7 @@ fn parse_fixed_fields(attrs: &Vec<syn::Attribute>) -> syn::Result<Vec<FixedField
 ///
 /// # Adding fixed properties
 ///
-/// Additional fields with fixed values can be added with the `object_fixed_fields` attribute on
+/// Additional fields with fixed values can be added with the `fixed_fields` attribute on
 /// the struct:
 ///
 /// ```
@@ -305,7 +373,7 @@ fn parse_fixed_fields(attrs: &Vec<syn::Attribute>) -> syn::Result<Vec<FixedField
 /// # use qualia_derive::ObjectShape;
 /// # use std::convert::{Infallible, TryFrom};
 /// #[derive(Debug, ObjectShape, PartialEq)]
-/// #[object_fixed_fields("kind" => "custom")]
+/// #[fixed_fields("kind" => "custom")]
 /// struct CustomShape {
 ///     width: i64,
 ///     height: i64,
@@ -322,10 +390,95 @@ fn parse_fixed_fields(attrs: &Vec<syn::Attribute>) -> syn::Result<Vec<FixedField
 ///     object!("kind" => "custom", "width" => 8, "height" => 11),
 /// );
 /// ```
-#[proc_macro_derive(
-    ObjectShape,
-    attributes(object_field, object_fixed_fields, object_rest_fields)
-)]
+///
+/// There is a helper method provided, [`q()`](`qualia::ObjectShape::q()`), which makes use of
+/// these fields. For example, for the above object shape, `CustomShape::q` returns a query for
+/// `"kind" = "custom"`.
+///
+/// # Accessing other properties
+///
+/// To set and fetch unlisted properties, an [`Object`](qualia::Object) field with the
+/// `rest_fields` attribute may be added.
+///
+/// ```
+/// # use qualia::{object, Object};
+/// # use qualia_derive::ObjectShape;
+/// # use std::convert::{Infallible, TryFrom};
+/// #[derive(Debug, ObjectShape, PartialEq)]
+/// struct CustomShape {
+///     width: i64,
+///     #[rest_fields]
+///     rest: Object,
+/// }
+///
+/// let shape: Object = CustomShape {
+///     width: 8,
+///     rest: object!("height" => "tall"),
+/// }
+/// .into();
+///
+/// assert_eq!(
+///     shape,
+///     object!("width" => 8, "height" => "tall"),
+/// );
+/// ```
+///
+/// # Getting ID of inserted object
+///
+/// The ID of the object can be retrieved from an `Option<i64>` field named `object_id`:
+///
+/// ```
+/// # use qualia::{object, Object};
+/// # use qualia_derive::ObjectShape;
+/// # use std::convert::{Infallible, TryFrom, TryInto};
+/// #[derive(Debug, ObjectShape, PartialEq)]
+/// struct CustomShape {
+///     object_id: Option<i64>,
+///     width: i64,
+/// }
+///
+/// let shape: CustomShape = object!(
+///     "object_id" => 42,
+///     "width" => 65,
+/// ).try_into().unwrap();
+///
+/// assert_eq!(
+///     shape,
+///     CustomShape { object_id: Some(42), width: 65 },
+/// );
+///
+/// // let shape2 = CustomShape { object_id: None, width: 11 };
+/// // store.insert_with_id(&mut shape2)?;
+/// // assert!(shape2.object_id.is_some());
+/// ```
+///
+/// # Accessing related objects
+///
+/// Often, objects contain references to other object's ID fields. If those objects have a defined
+/// `ObjectShape`, then helper methods to fetch them can be generated with the `related`
+/// attribute:
+///
+/// ```
+/// # use qualia::{object, Object};
+/// # use qualia_derive::ObjectShape;
+/// # use std::convert::{Infallible, TryFrom};
+/// #[derive(Debug, ObjectShape, PartialEq)]
+/// struct ShapeGroup {
+///     object_id: Option<i64>,
+///     name: String,
+/// }
+///
+/// #[derive(Debug, ObjectShape, PartialEq)]
+/// struct CustomShape {
+///     #[related(ShapeGroup)]
+///     shape_group_id: i64,
+///     width: i64,
+/// }
+///
+/// // if let Some(group) = custom_shape.fetch_shape_group(&store)? {;
+/// //     ...
+/// ```
+#[proc_macro_derive(ObjectShape, attributes(field, fixed_fields, rest_fields, related))]
 pub fn derive_object_shape(input: TokenStream) -> TokenStream {
     let parsed_struct = parse_macro_input!(input as DeriveInput);
     let orig_type_name = parsed_struct.ident;
@@ -362,14 +515,40 @@ pub fn derive_object_shape(input: TokenStream) -> TokenStream {
 
     let (parsed_fields, rest_field_ident) = try_or_error!(parse_fields(&named_fields));
 
+    let object_shape_with_id_impl =
+        if let Some(field) = parsed_fields.iter().find(|f| f.name == "object_id") {
+            let field_ident = field.ident.clone();
+
+            quote!(
+                impl qualia::ObjectShapeWithId for #orig_type_name {
+                    fn get_object_id(&self) -> Option<i64> {
+                        self.#field_ident
+                    }
+
+                    fn set_object_id(&mut self, object_id: i64) {
+                        self.#field_ident = Some(object_id);
+                    }
+                }
+            )
+        } else {
+            quote!()
+        };
+
     let mut field_names = Vec::new();
     let mut field_idents = Vec::new();
     let mut field_accessors = Vec::new();
+    let mut field_inserters = Vec::new();
+    let mut field_related_impls = Vec::new();
 
     for f in parsed_fields.into_iter() {
         field_names.push(f.name);
         field_idents.push(f.ident);
         field_accessors.push(f.accessor);
+        field_inserters.push(f.inserter);
+
+        if let Some(related_impl) = f.related_impl {
+            field_related_impls.push(related_impl);
+        }
     }
 
     let rest_field_try_from = if let Some(ref rest_field_ident) = rest_field_ident {
@@ -426,16 +605,33 @@ pub fn derive_object_shape(input: TokenStream) -> TokenStream {
             fn into(self) -> qualia::Object {
                 use qualia::{object, Object};
 
+                #[allow(unused_mut)]
                 let mut result = object!(
                     #(#fixed_field_names => #fixed_field_values),*
-                    #(#field_names => self.#field_idents),*
                 );
+                #(#field_inserters)*
                 #rest_field_into
 
                 result
             }
         }
 
-        impl qualia::ObjectShape for #orig_type_name {}
+        impl qualia::ObjectShape for #orig_type_name {
+            fn q() -> qualia::query_builder::QueryBuilder {
+                qualia::Q
+                #(
+                    .equal(
+                        #fixed_field_names,
+                        #fixed_field_values,
+                    )
+                )*
+            }
+        }
+
+        #object_shape_with_id_impl
+
+        impl #orig_type_name {
+            #(#field_related_impls)*
+        }
     ).into()
 }
